/*
 * Copyright (c) 2022–2024 DTB <trinity@trinity.moe>
 * SPDX-License-Identifier: AGPL-3.0-or-later
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see https://www.gnu.org/licenses/.
 */

use std::{
	env::args,
	io::Error,
	os::unix::process::CommandExt,
	process::{ Command, ExitCode }
};

extern crate getopt;
use getopt::GetOpt;

extern crate strerror;
use strerror::StrError;

extern crate sysexits;
use sysexits::{ EX_UNAVAILABLE, EX_USAGE };

fn error(p: &str, n: &str, e: Error) -> ExitCode {
	eprintln!("{}: {}: {}", p, n, e.strerror());
	ExitCode::from(EX_UNAVAILABLE as u8)
}

fn usage(s: &str) -> ExitCode {
	eprintln!("Usage: {} command [argument...]");
	ExitCode::from(EX_USAGE as u8)
}

fn main() -> ExitCode {
	let argv = args().collect::<Vec<String>>();
	let mut optind = 0;

	let mut arg0: Option<String> = None;
	let mut dir = String::from(".");

	let mut cmd;

	while let Some(opt) = argv.getopt("C:n") {
		match opt.opt() {
			Ok("C") => dir = opt.arg().unwrap(),
			Ok("n") => arg0 = Some(opt.arg().unwrap()),
			_ => { return usage(&argv[0]); }
		}
		optind = opt.ind();
	}

	if argv.len() - optind < 1 { return usage(&argv[0]); }

	cmd = Command::new(argv[optind].clone()).current_dir(argv[1].clone());

	if let Some(a) = arg0 { cmd.arg0(a); }

	if argv.len() - optind > 2 {
		let cmdargs = argv
			.iter()
			.clone()
			.skip(optind + 1)
			.collect::<Vec<&String>>();
		cmd.args(cmdargs);
	}

	match cmd.spawn() {
		Ok(mut child) => match child.wait() {
			Ok(status) => {
				match status.code() {
					Some(code) => ExitCode::from(code as u8),
					None => {
						eprintln!("{}: {}: process terminated by signal",
							argv[0], argv[2]);
						ExitCode::FAILURE
					}
				}
			},
			Err(e) => error(&argv[0], &argv[2], e)
		},
		Err(e) => error(&argv[0], &argv[2], e)
	}
}
